#!/usr/bin/env perl
use strict;
use warnings;
use lib './gen-perl/';

package Thrift::Socket;
use IO::Select;

sub setHandle {
    my ($self, $sock) = @_;
    $self->{handle} = IO::Select->new( $sock );
}

package Thrift::ServerSocket;
use base qw/Class::Accessor::Fast/;

__PACKAGE__->mk_accessors(qw/port handle/);

use IO::Socket;
use Thrift::Socket;

sub new {
    my $class = shift;
    my $self = $class->SUPER::new;
    $self->port( shift || 9090 );
    bless $self, $class;
}

sub listen {
    my $self = shift;
    $self->handle(
        IO::Socket::INET->new(
            LocalPort => $self->port,
            Listen    => SOMAXCONN,
            Proto     => 'tcp',
            Reuse     => 1,
        )
    );
}

sub accept {
    my $self = shift;

    if (defined $self->handle) {
        my $sock = $self->handle->accept;

        my $trans = Thrift::Socket->new;
        $trans->setHandle( $sock );

        return $trans;
    }

    return;
}

package Thrift::SimpleServer;
use base qw/Class::Accessor::Fast/;

use Thrift::BufferedTransport;
use Thrift::BinaryProtocol;

__PACKAGE__->mk_accessors(qw/processor server_transport/);

sub new {
    my ($class, @args) = @_;
    my $self  = $class->SUPER::new;

    $self->processor( $args[0] );
    $self->server_transport( $args[1] );

    bless $self, $class;
}

sub serve {
    my $self = shift;
    $self->server_transport->listen;
    while (1) {
        my $client = $self->server_transport->accept or die Thrift::Exception->new($!);

        ## FIXME: use a factory instead of a concrete class.
        my $trans = Thrift::BufferedTransport->new($client, 10240, 10240);
        my $prot  = Thrift::BinaryProtocol->new( $trans );

        eval {
            while (1) { $self->processor->process( $prot, $prot ) }
        };
        if (my $e = $@) {
            ## NOTE: These exceptions are not implemented yet.
            if ($e->isa('Thrift::TTransportException') or $e->isa('Thrift::TProtocolException')) {
                warn $e->{message};
            } else {
                $trans->close;
            }
        }
    }
    $self->server_transport->close;
}

package HelloHandler;
use Hello;
use base qw/HelloIf/;

sub new { bless {}, shift }

sub hello {
    my ($self, $name) = @_;
    return sprintf "Hello, %s", $name;
}

package main;

my $processor = HelloProcessor->new( HelloHandler->new );
my $transport = Thrift::ServerSocket->new(9090);

my $server = Thrift::SimpleServer->new( $processor, $transport );
$server->serve;
